<div class="flex flex-col gap-4">
    <p>You were successfully logged into Wialon! No further action is required and you may leave this page at any time.</p>
    <a href="{% url 'terminusgps_notifications:dashboard' %}" hx-boost="true" class="w-full cursor-pointer rounded border border-terminus-red-900 bg-terminus-red-800 p-2 text-center text-white transition-colors duration-300 ease-in-out hover:bg-terminus-red-400">Go To Dashboard</a>
</div>
