<form class="flex flex-col gap-2" id="id_trigger_parameters" hx-post="{% url 'terminusgps_notifications:trigger parameters' %}" hx-target="this" hx-swap="outerHTML" hx-include="#id_t">
    {% csrf_token %}
    {% for field in form %}
    <section class="flex flex-col gap-2">
        {{ field }}
        <label class="order-first peer-invalid:text-red-800" for="{{ field.id_for_label }}">{{ field.label }}</label>
        <p class="text-sm peer-invalid:text-red-600" id="{{ field.id_for_label }}_helptext">{{ field.help_text }}</p>
        <div class="animate-jiggle font-italic">{{ field.errors }}</div>
    </section>
    {% endfor %}
    <button type="submit" class="flex w-full cursor-pointer items-center justify-center gap-2 rounded border border-terminus-red-900 bg-terminus-red-800 p-2 text-center text-white transition-colors duration-300 ease-in-out hover:bg-terminus-red-400 animate-pulse">Save Parameters</button>
</form>
