from .customers import (
    AccountView,
    CustomerStatsView,
    CustomerSubscriptionCreateView,
    DashboardView,
    NotificationsView,
    SubscriptionView,
    WialonCallbackView,
    WialonLoginView,
)
from .main import HomeView
from .notifications import (
    WialonNotificationCreateView,
    WialonNotificationDeleteView,
    WialonNotificationDetailView,
    WialonNotificationListView,
    WialonNotificationTriggerParametersFormSuccessView,
    WialonNotificationTriggerParametersFormView,
    WialonNotificationTriggerSelectFormView,
    WialonNotificationUnitSelectFormView,
    WialonNotificationUpdateView,
)
