from django.apps import AppConfig


class TerminusgpsNotificationsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "terminusgps_notifications"
    verbose_name = "Terminus GPS Notifications"
